//  *****************************************************************************************************************
//  *                                                                                                               *
//  *                                         SpikenzieLabs.com                                                     *
//  *                                                                                                               *
//  *                                         I2C LCD Driver v3                                                     *
//  *                                                                                                               *
//  *                                     Demo Program I2C Interface                                                *
//  *                                                                                                               *
//  *****************************************************************************************************************
//
// BY: MARK DEMERS 
// Dec 2010
// VERSION: 3.b
//
// Changes:
// This version of the PCB has a transitor connected to button pin 5 which may be used to control the LCD backlighting.
// While used for backlighing control button number five can not be used for button input.
// This code has been modified to use backlighting control.
// If you would like to use button number five for input, use the older version of the I2C interface sketch and omit
// the transistor and the rightmost 10k resistor from the I2C interface. If you need to power a backlight while using
// pin five for input, jumper the solder jumper on the back of the PCB.
//
// DESCRIPTION:
// This is a demo program to specifically drive the I2C LCD v3 "back-pack" from SpikenzieLabs.com using I2C mode.
// The sections or single lines of code that are marked as REQUIRED are, as the name implies, required to make
// the device work. You may either build your projects around this demo code or copy and paste it into your 
// own.
//
// HOOK-UP:
//  SIGNAL     I2C LCD BOARD    -    ARDUINO
//  Ground          GND                 GND
//   VCC            5V                  5V
//   SDA**          SDA               Analog 4
//   SCL**          SCL               Analog 5
//
//   The buttons (if used) are wired with a normally open (NO) switch to ground. (Pressing the button will contact the 
//   ground signal).
//
// ** I2C communications require a pull-up resistor on both the SCL and SDA lines. There are 4.7k ohm resistors 
//    on both the SDA and SCL line, pulling them up to VCC. Even if you are using other I2C devices
//    use only one set of pull-up resistors. If you already have pull-up resistors in your project omit the two 4.7k
//    ohm resistors
//
//  
//
// USAGE:
// 1. Disconnect the power and hook-up the Arduino and the I2C Board as described above.
// 2. Set your I2C address by using solder to jumper the pads for A2,A1 or A0
//    (Up to 7 I2C Boards and be used on the same I2C bus.)
//    See below in the define section for this line
//                     #define		MCP23017	B00100xxx 
//                                                       the "xxx" is your board's changeable I2C Address 
// 3. Power the board and upload the sketch. 
// 4. You should see a message and a counter on your LCD. If you hooked up buttons and are pressing one, you should 
//    also see that message. You may need to adjust the contrast POT (blue square on PCB) to see the display properly. 
//    This demo will also put the LCD backlighing to "sleep" ie turn off, after about 20 seconds until you press a 
//    button again, which will turn it on again for another 20 seconds.
//
//
//
// LEGAL:
// This code is provided as is. No guaranties or warranties are given in any form. It is up to you to determine
// this codes suitability for your application.
//




#include <Wire.h>   // REQUIRED                                                   Used for the I2C Communications

//*******************************************************************************************************************
// REQUIRED							                                Define Statements 
//*******************************************************************************************************************

// Port Expander
#define		MCP23017	B00100000	//MCP23017 I2C Address was B01000000
//                                    xxx <- These are the I2C address bits

#define		IOCON		0x0A		// MCP23017 Config Reg.

#define		IODIRA		0x00		// MCP23017 address of I/O direction
#define		IODIRB		0x01		// MCP23017 1=input

#define		IPOLA		0x02		// MCP23017 address of I/O Polarity
#define		IPOLB		0x03		// MCP23017 1= Inverted

#define		GPIOA		0x12		// MCP23017 address of GP Value
#define		GPIOB		0x13		// MCP23017 address of GP Value

#define		GPINTENA	0x04		// MCP23017 IOC Enable
#define		GPINTENB	0x05		// MCP23017 IOC Enable

#define		INTCONA		0x08		// MCP23017 Interrupt Cont 
#define		INTCONB		0x09		// MCP23017 1= compair to DEFVAL(A or B) 0= change

#define		DEFVALA		0x06		// MCP23017 IOC Default value
#define		DEFVALB		0x07		// MCP23017 if INTCONA set then INT. if diff. 

#define		GPPUA		0x0C		// MCP23017 Weak Pull-Ups
#define		GPPUB		0x0D		// MCP23017 1= Pulled HIgh via internal 100k

// LCD
#define		ClrLCD		0x01		// clear the LCD
#define		CrsrHm		0x02		// move cursor to home position
#define		CrsrLf		0x10		// move cursor left
#define		CrsrRt		0x14		// move cursor right
#define		DispLf		0x18		// shift displayed chars left
#define		DispRt		0x1C		// shift displayed chars right
#define		DDRam		0x80		// Display Data RAM control
#define		ddram2		0xC0		// 9th position of display (not in next 

#define		RS_pin		B00000100
#define		RW_pin		B00000010
#define		E_pin		B00000001


//*******************************************************************************************************************
//								                                   VARIABLE INITS
//*******************************************************************************************************************

byte LCDCONT    = 0;        // REQUIRED
byte button = 0;            // REQUIRED
int  i  =0;                 // REQUIRED

int onoffval =0;    

boolean backlightStatus = false;    // Back Lighting on or off
  
unsigned long Starttime;            // Timer for Dispay Sleep
unsigned long Counttime;            // Timer for Dispay Sleep
#define sleeptimmer 20000           // Delay before display backlight sleep

int buttonPress    =0;
int count  = 0;

// SIZER        "0123456789012345678901234567890123456789"
char mess[16] = "I2C LCD ";  // Hello style it's working message
char mess1[16] ="Button: ";  // Part of the "a button is being pressed" message
char NUMB[6] =  "00000";

//*******************************************************************************************************************
// REQUIRED								                                    SET-UP 
//*******************************************************************************************************************

void setup()
{
  Wire.begin();                                                        // join i2c bus (address optional for master)
  portexpanderinit();
  delay(200);
  lcdinit();
  delay(500);
  LCDcmd(ClrLCD);
    
  backlightStatus = true;
  backlight(backlightStatus);
  
    Starttime = millis();
}

//*******************************************************************************************************************
//								                                        MAIN LOOP
//*******************************************************************************************************************

void loop()
{
  
  Counttime = millis();
  Counttime = Counttime - Starttime;
  
  if(Counttime > sleeptimmer)                                          // Example of timed sleep for LCD backlighting
  {
    backlightStatus = false;
    backlight(backlightStatus);
  }
  
  
  
  for (i=0;i<8;i++)
  {
    LCDwr(mess[i]);                                                    // Print-out basic message
  }
  dispnumb(count);                                                     // Print-out ounter value to display
  count = count +1;
  
 checkbutton();                                                        // Check if Option an button is pressed
  if (int(button)>0)
  {
    LCDcmd(ddram2);                                                    // Line 2 of 16x2 LCD
    for (i=0;i<8;i++)
    {
      LCDwr(mess1[i]);                                                 // Print out button pressed message with number
    }
 
    dispnumb(int(button));                                             // Reset display sleep timer
    Starttime = millis();
    backlightStatus = true;
    backlight(backlightStatus);
  }
  
  
  delay(1000);                                                         // Delay and do it again
  LCDcmd(ClrLCD);


}

//*******************************************************************************************************************
//								                         Functions and Subroutines
//*******************************************************************************************************************


//*******************************************************************************************************************
// REQUIRED								                                 I2C TX RX
//*******************************************************************************************************************
void I2C_TX(byte device, byte regadd, byte tx_data)                              // Transmit I2C Data
{
  Wire.beginTransmission(device);
  Wire.send(regadd); 
  Wire.send(tx_data); 
  Wire.endTransmission();
}

void I2C_RX(byte devicerx, byte regaddrx)                                       // Receive I2C Data
{
  Wire.beginTransmission(devicerx);
  Wire.send(regaddrx); 
  Wire.endTransmission();
  Wire.requestFrom(int(devicerx), 1);   

  byte c = 0;
  if(Wire.available())
  {
    byte c = Wire.receive();    
    c = c & 0x7F;                                                               // New: removed button 5 (now used for backlight)
    button = c >>3;
  }
}

//*******************************************************************************************************************
// REQUIRED								                                  LCD INIT
//*******************************************************************************************************************

void lcdinit()
{
  // Only used with port expander
  LCDCONT = RS_pin | E_pin | onoffval;
  I2C_TX(MCP23017,GPIOB,LCDCONT);

  delay(50);
  LCDcmd(B00110000);                 // Standard Hitachi initialization for 8-bit mode form sepc sheets
  delay(60);
  LCDcmd(B00110000);    
  delay(60);
  LCDcmd(B00110000);      
  delay(60);
  LCDcmd(B00111000);      
  delay(60);
  LCDcmd(B00001000);        
  delay(60);
  LCDcmd(ClrLCD);          
  delay(60);
  LCDcmd(B00000110); 	
  delay(60);
  LCDcmd(B0001100);        	
  delay(60);
}

//*******************************************************************************************************************
// REQUIRED								                         PORTEXPANDER INIT
//*******************************************************************************************************************
void portexpanderinit()
{
  // --- Set I/O Direction
  // old I2C_TX(MCP23017,IODIRB,B11111000);
  I2C_TX(MCP23017,IODIRB,B01111000);                              // New: removed button 5 (now used for backlight)
  //
  I2C_TX(MCP23017,IODIRA,B00000000);
  //  --- Set I/O Polarity
  I2C_TX(MCP23017,IPOLA,B00000000);
  //I2C_TX(MCP23017,IPOLB,B11111000);
  I2C_TX(MCP23017,IPOLB,B01111000);
  //  --- Set ALL Bits of GPIOA
  I2C_TX(MCP23017,GPIOA,B00000000);
  // --- Set Weak Pull-Up on Bits 7 of GPIOB
 // old I2C_TX(MCP23017,GPPUB,B11111000);
  I2C_TX(MCP23017,GPPUB,B01111000);                                // New: removed button 5 (now used for backlight)
  // --- Set Default on Bits 7 of GPIOB
  I2C_TX(MCP23017,DEFVALB,B00000000);
  // --- Set Use Default on Bits 7 of GPIOB
  I2C_TX(MCP23017,INTCONB,B10000000);
  // --- Set IOC on Bits 7 of GPIOB
  I2C_TX(MCP23017,GPINTENB,B10011000);
  // --- Set active low of int pin
  I2C_TX(MCP23017,IOCON,B00110000);
}

//*******************************************************************************************************************
// REQUIRED								                       LCD WRITE / COMMAND
//*******************************************************************************************************************
void LCDcmd(byte cmdlcd)
{
  LCDCONT =0;                                        // was bcf RS_pin
  LCDCONT = LCDCONT | onoffval;
  I2C_TX(MCP23017,GPIOB,LCDCONT);
  LCDwr(cmdlcd);
  delay(20);
}

void LCDwr(byte lcdChar)
{
  I2C_TX(MCP23017,GPIOA,lcdChar);
  LCDCONT = LCDCONT | E_pin | onoffval;              // If RS is set then it stays set
  delay(2);
  LCDCONT = LCDCONT | onoffval;
  I2C_TX(MCP23017,GPIOB,LCDCONT);
  LCDCONT = RS_pin | onoffval;
  I2C_TX(MCP23017,GPIOB,LCDCONT);
}


//*******************************************************************************************************************
//								                   SEND DECIMAL NUMBER TO DISPLAY
//*******************************************************************************************************************
void dispnumb(int numbvar)
{
  boolean leadzero = false;                                        // Show leading zero or not
  int numbdigits = 5;                                              // Number of digits in your number (1 to 5)
    
  boolean Digitflag = false;                                       // Used internally when ok to show numbers

  int TENTHUS= abs(numbvar/10000);
  int THUS= abs((numbvar- (TENTHUS*10000))/1000);
  int HUNDS= abs((numbvar- (TENTHUS*10000)-(THUS*1000))/100);
  int TENS = abs((numbvar - (TENTHUS*10000)-(THUS*1000)-(HUNDS*100))/10);
  int ONES = abs(numbvar- (TENTHUS*10000)-(THUS*1000)-(HUNDS*100)-(TENS*10));

  NUMB[0] =  TENTHUS+48;
  NUMB[1] =  THUS+48;
  NUMB[2] =  HUNDS+48;                                                        // +48 for ASCII conversion
  NUMB[3] =  TENS+48;
  NUMB[4] =  ONES+48;

  for(i=(5-numbdigits);i<5;i++)
  {
    if(Digitflag || leadzero || (NUMB[i]-48))
    {
    LCDwr(NUMB[i]);                                                           // Print-Out 1 to 5 digit number at cursor
    Digitflag = true;
    }
  }
}

//*******************************************************************************************************************
//								                 READ IF BUTTONS ARE BEING PRESSED
//*******************************************************************************************************************
void checkbutton()                            // Use serial monitor at 9600bps to see buttons that are pressed
{
  I2C_RX(MCP23017,GPIOB);
  buttonPress = int(button);
  switch (buttonPress)                        
  {
  case 1:
  button = 1;
    break;
    
  case 2:
  button = 2;
    break;
    
  case 4:
  button = 3;
    break;
    
  case 8:
  button = 4;
    break;
    
   // case 16:                                       // There is no button "5" while using firmware controled back-lighting
   //   button = 5;
   //  break; 
  }
}

void backlight(boolean onoff)
{
  onoffval =0;
  LCDCONT = LCDCONT & 0x7F;                           // Used to mask the port expander port used for LCD control
  if(onoff == true)
  {
   onoffval =0x80;
  }
  
  LCDCONT = LCDCONT | onoffval;
  I2C_TX(MCP23017,GPIOB,LCDCONT);
}

